#if 0 /*
# -----------------------------------------------------------------------
# __init__.py - MainMenuItem providing a mailreader for freevo
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: __init__.py,v 1.13 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: __init__.py,v $
# Revision 1.13  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# freevo modules
import menu, plugin, config, rc, watcher
from item       import Item
from gui        import ConfirmBox
from event      import *
from mailreader import *
from mailwriter import *

plugin.activate('mail.watcher.Mailwatcher')
mwatch = plugin.getbyname('Mailwatcher')



class MailItem(Item):
    """
    this class defines actions for an mailitem
    """
    def __init__(self,parent=None):
        Item.__init__(self,parent)
        self.parent = parent
        self.type='email'

    def actions(self):
        """
        return a list of actions for this item
        """
        items = [ (self.readpopup,   _('Read email')      ),
                  (self.replyto,     _('Reply to sender') ) ]

        if 'DEL' in mwatch.caps:
            items += [ (self.deletemail, _('Delete email') ) ]

        if 'FLAG' in mwatch.caps:
            items += [ (self.flagread,   _('Mark as read')   ),
                       (self.flagunread, _('Mark as unread') ) ]
        return items

    def replyto(self, arg=None, menuw=None):
        mail = mwatch.action('READ', self.header)
        MailSender(mail, menuw)

    def flagunread(self,arg=None,menuw=None):
        """
        flags a message as read
        """
        try:
            mwatch.action('MARK UNREAD', self.header)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
        menuw.back_one_menu(menuw=menuw)

    def flagread(self,arg=None,menuw=None):
        """
        flags a message as unread
        """
        try:
            mwatch.action('MARK READ', self.header)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
        menuw.back_one_menu(menuw=menuw)

    def deletemail(self,arg=None,menuw=None):
        """
        deletes an email from the box
        """
        self.menuw = menuw
        message=_('Are you sure you want to delete this email?')
        ConfirmBox(text=message, default_choice=1, handler=self._delMail,
                   handler_message=_('Deleting...')).show()

    def _delMail(self):
        try:
            mwatch.action('DELETE', self.header)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
        self.menuw.back_one_menu(menuw=self.menuw)
        self.menuw = None

    def readpopup(self, arg=None, menuw=None):
        """
        calls the mailreader to view an email
        """
        mail = None

        try:
            mail = mwatch.action('READ',self.header)
        except watcher.Mailwatcher.error, err:
            err.alert.show()

        if mail:
            MailReader(mail,menuw)



class BoxItem(Item):
    def __init__(self,parent=None, skin=None):
        if skin:
            Item.__init__(self, parent, skin_type='mail')
        else:
            Item.__init__(self, parent)

        self.type='mailbox'

    def actions(self):
        """
        return a list of actions for IMAP,POP and mailbox
        """
        items = [ (self.openmbox , _('Open mailbox') ) ]

        if 'FLAG' in mwatch.caps:
            items.append((self.markAll ,  _('Mark all mails read') ))

        if 'SUBS' in mwatch.caps:
            items += [ (self.unsubscribe, _('Unsubscribe from folder') ),
                       (self.subscribe,   _('Subscribe to new folder') ) ]
        return items

    def markAll(self, arg=None, menuw=None):
        try:
            mwatch.action('MARK ALL READ',self.folder)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
        menuw.back_one_menu(menuw=menuw)

    def unsubscribe(self, arg=None, menuw=None):
        """
        make it possible for user to unsubscribe from folders
        """
        self.menuw = menuw
        message=_('Are you sure you want to unsubscribe from %s?') % self.folder
        ConfirmBox(text=message, handler=self._unsubscribe,
                   default_choice=1, handler_message=_('Unsubscribing...')).show()

    def _unsubscribe(self):
        try:
            mwatch.action('UNSUBSCRIBE',self.folder)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
        self.menuw.back_one_menu(menuw=self.menuw)
        self.menuw = None

    def subscribe(self, arg=None, menuw=None):
        """
        make it possible for user to subscribe to new folders
        """
        sub_items = []
        try:
            names = mwatch.action('GETUNSUB')
            for n in names:
                sub = SubscribeItem(self.parent)
                sub.name = n
                sub.mbox = n
                sub_items += [ sub ]

            sub_menu = menu.Menu('mail', sub_items, reload_func=self.reload)
            menuw.pushmenu(sub_menu)
            menuw.refresh()
        except watcher.Mailwatcher.error, err:
            err.alert.show()

    def openmbox(self, arg=None, menuw=None):
        """
        opens a mailbox
        """
        self.items = self.build(menuw)
        item_menu = menu.Menu(self.name, self.items,
                              item_types = 'mail',
                              reload_func = self.reload)
        item_menu.skin_force_text_view = False
        menuw.pushmenu(item_menu)
        self.menuw = menuw

    def build(self,menuw=None):
        """
        builds the next tier
        """
        mail_items = []
        try:
            mails = mwatch.getHeaders(self.folder)
        except watcher.Mailwatcher.error, err:
            err.alert.show()
            # try again in offline mode (get cached mails)
            try:
                mails = mwatch.getHeaders(self.folder, offline=True)
            except watcher.Mailwatcher.error, err:
                # never ever got a connection
                mails = []
                err.alert.show()

        if mails:
            for a in mails:
                mail = MailItem(self.parent)
                mail.header = a

                # info
                mail.name = a.Subject
                mail.subject = a.Subject
                mail.date = a.Date
                mail.fromname = a.FromName
                mail.fromaddr = a.FromAddr
                mail.toname = a.ToName
                mail.toaddr = a.ToAddr

                if a.Flag=='new':
                    mail.type = 'newemail'
                else:
                    mail.type = 'email'

                mail_items += [ mail ]

        if menuw:
            self.menuw = menuw

        return mail_items

    def reload(self):
        stack = len(self.menuw.menustack) - 1
        menu = self.menuw.menustack[stack]
        sel = menu.choices.index(menu.selected)
        new_choices = self.build()
        num = len(new_choices)

        # select an item
        if num==0:
            menu.selected = None
        else:
            if (num-1) < sel: s = num-1
            else:             s = sel
            menu.selected = new_choices[s]

        menu.choices = new_choices

        return menu



class FolderItem(Item):
    """
    this is the item for the main menu and creates the list
    of subscribed mailboxes.
    Used when using IMAP or when having set multiple mailfiles
    """
    def __init__(self, parent):
        Item.__init__(self, parent, skin_type='mail')

    def actions(self):
        """
        return a list of actions for this item
        """
        return [ ( self.create_mails_menu , _('Open mailfolder') ) ]

    def create_mails_menu(self, arg=None, menuw=None):
        self.menuw = menuw
        items = self.build()
        if items:
            item_menu = menu.Menu(_('Mailfolders'), items,
                                  item_types='mail', reload_func=self.reload)
            item_menu.skin_force_text_view = False
            menuw.pushmenu(item_menu)

    def build(self):
        """
        builds the next tier
        """
        box_items = []
        name = []

        try:
            name = mwatch.getFolders()
            if name:
                for n in name:
                    box = BoxItem()
                    box.newmails, box.mails = mwatch.action('STAT',n)
                    box.folder = n
                    box.name = n
                    box_items += [ box ]
        except watcher.Mailwatcher.error ,err:
            err.alert.show()

        return box_items

    def reload(self):
        stack = len(self.menuw.menustack) - 1
        menu = self.menuw.menustack[stack]
        sel = menu.choices.index(menu.selected)
        new_choices = self.build()
        num = len(new_choices)

        # select an item
        if num==0:
            menu.selected = None
        else:
            if (num-1) < sel: s = num-1
            else:             s = sel
            menu.selected = new_choices[s]

        menu.choices = new_choices

        return menu



class SubscribeItem(Item):
    """
    Subscribe to new folders
    used only by the imap protocol
    """
    def actions(self):
        return [ ( self.subscribe , _('Subscribe to folder') ) ]

    def subscribe(self, arg=None, menuw=None):
        try:
            mwatch.action('SUBSCRIBE', self.mbox)
            menuw.back_one_menu(menuw=menuw)
            menuw.delete_menu(menuw=menuw)
            menuw.refresh()

        except watcher.Mailwatcher.error, err:
            err.alert.show()



class PluginInterface(plugin.MainMenuPlugin):
    """
    Plugin which provides a mailreader for different mail-protocols.
    Activate with:
      - plugin.activate('mail',level=x) and
      - plugin.activate('idlebar.newmail.newmail',level=10)
        if you want to use the idlebar to show new mails

    Configuration:
     MAIL_TYPE='type'  - Choose between mbox, maildir, imap and pop3
     MAIL_INTERVAL=300 - How often (in seconds) should it check for
                         new email
     imap/pop3:
       MAIL_HOST='hostname.com' - Hostname of the server
       MAIL_USER='yourusername' - Username
       MAIL_PASS='yourpass'     - Password

       MAIL_SSL=0/1 - Should work in python 2.3+
                      Use SSL for IMAP4 connections.
                      Does not work because of default socket
                      timing set in utils/pymetar.py. Remove
                      setting at top if you wish to use this
                      option.

      maildir/mbox:
       MAIL_FILE=['/path/to/file_or_dir',
                  '/path/to/other/file_or_dir',
                  ... ]
      sending mail:
       MAIL_SMTP = { 'from'  : 'user@dom.com',
                     'name'  : 'Firstname Lastname',
                     'sig'      : 'If you want a bad quote (like this one)',
                     'reply-to' : 'user@dom.com',
                     'server'   : 'server.dom.com',
                     'port'     : 25,
                     'username' : 'username',
                     'password' : 'password'
                   }
       One can omit username and password if the
       server does not require login information,
       port if it uses default port, reply-to
       if this is not in use and sig if this
       seems redundant.
    """
    def actions(self):
        return [ ( self.test, _('test') ) ]

    def items(self, parent):
        if mwatch.type in ['maildir','mbox'] and len(config.MAIL_FILE) == 1:
            box = BoxItem(parent, 1)
            box.folder = config.MAIL_FILE[0]
            return [ box ]
        elif mwatch.type == 'pop3':
            box = BoxItem(parent, 1)
            box.folder = 'Pop3 mail'
            return [ box ]
        else:
            return [ FolderItem(parent) ]

    def config(self):
        return [ ('MAIL_TYPE', 'IMAP', 'Choose between IMAP/POP3/MBOX'),
                 ('MAIL_HOST', 'localhost:143', 'IMAP server address - host[:port]'),
                 ('MAIL_USER', 'locouser', 'IMAP username'),
                 ('MAIL_PASS', 'pass12345', 'IMAP password'),
                 ('MAIL_INTERVAL', 300, 'Check interval in seconds'),
                 ('MAIL_FILE', "[ '/var/spool',...]", 'file or dirname for mbox/maildir'),
                 ('MAIL_SSL', 0, 'Use IMAP over ssl') ]
