#if 0 /*
# -----------------------------------------------------------------------
# mail.py - superclasses for mail
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: mail.py,v 1.9 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: mail.py,v $
# Revision 1.9  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Todo:
# - Remove MailProtocol if it's not needed anymore
# - Cleanup the naming of Header and Message
#
# NOTES:
# - required
#   login()
#   logout()
#   getFolders()
#   getHeaders(foldername)
#   getUIDList(foldername)
#   getHeader(foldername, uid)
#   getMessage(header)
#
# - optional
#   flagMessage(header,action='UNREAD|READ')
#   delMessage(header)
#
# - imap specific
#   unsubscribe(arg)
#   subscribe(arg)
#   getUnsubscribed()
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# freevo modules
import config

def get_protocol():
    type = config.MAIL_TYPE.lower()
    # get the correct protocol
    if type == 'pop3':
        import pop3
        return pop3.Pop3()
    elif type == 'imap':
        import imap
        return imap.Imap()
    elif type == 'maildir':
        import maildir
        return maildir.Maildir()
    import mbox
    return mbox.Mbox()



class MailProtocol:
    """
    Superclass for all protocols
    This defines the minimum required set of
    functions needed for a protocol

    caps:
         DEL   : uses delMessage()
         FLAG  : uses flagMessage()
         PURGE : uses purge()
         SUBS  : uses subscribe() / unsubscribe
         NOFOLD  : not FolderItem
    """
    def __init__(self):
        self.type = config.MAIL_TYPE.lower()
        self.caps = []

        if self.type in ['imap','pop3']:
            self.username = config.MAIL_USER
            self.password = config.MAIL_PASS
            self.host     = config.MAIL_HOST

            try:    self.SSL = config.MAIL_SSL
            except: self.SSL = 0

        elif self.type in [ 'mbox', 'maildir']:
            self.file = config.MAIL_FILE

    def __del__(self):
        print 'del'
        self.logout()

    def login(self):
        return 'OK'

    def logout(self):
        return 'OK'

    def getFolders(self):
        return self.file

    def Sort(self, headers):
        headers.sort( lambda l, o: cmp(o.Sort, l.Sort))
        return headers



class BaseHeader:
    """
    what our headers should look like
    """
    def __init__(self):
        self.Subject = '(No subject)'
        self.Date = 'No date found'
        self.Sort = 0
        self.FromAddr = 'no address found'
        self.FromName = 'no name found'
        self.ToAddr = 'no address found'
        self.ToName = 'no name found'
        self.Flag = 'seen'
        self.UID = ''
        self.Folder = ''
        self.Optional = None

    def _unicode(self):
        # try to fix unicode stuff
        self.Subject = Unicode(self.Subject)
        self.Date = Unicode(self.Date)
        self.FromAddr = Unicode(self.FromAddr)
        self.FromName = Unicode(self.FromName)
        self.ToAddr = Unicode(self.ToAddr)
        self.ToName = Unicode(self.ToName)

    def unicode(self):
        self._unicode()

class BaseMessage(BaseHeader):
    """
    what our messages should look like
    """
    def __init__(self,header=None):
        self.BodyText    = ''
        self.MediaAttach = []

        BaseHeader.__init__(self)

        if header:
            self.setattr(header)

    def __unicode(self):
        self.BodyText = Unicode(self.BodyText)

    def unicode(self):
        self._unicode()
        self.__unicode()

    def setattr(self,h):
        self.Subject = h.Subject
        self.Date = h.Date
        self.Sort = h.Sort
        self.FromAddr = h.FromAddr
        self.FromName = h.FromName
        self.Flag = h.Flag
        self.ToName = h.ToName
        self.ToAddr = h.ToAddr
        self.UID = h.UID
        self.Folder = h.Folder
        self.Optional = h.Optional
