#if 0 /*
# -----------------------------------------------------------------------
# mailreader.py - reader for the mail
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: mailreader.py,v 1.11 2004/05/19 10:59:12 viggo Exp $
# -----------------------------------------------------------------------
# $Log: mailreader.py,v $
# Revision 1.11  2004/05/19 10:59:12  viggo
# Fix a skin change issue.
# Don't use colorkey on the surfaces
#
# Revision 1.10  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Notes:
# Todo: o Take a look at drawing some indicators
#         for the scrollarea.
#       o Add support for multipart messages
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2002 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# python modules
import pygame, os

# freevo modules
import rc, skin, osd, config
from event      import *
from mailwriter import MailSender

skin = skin.get_singleton()
osd  = osd.get_singleton()



class MailReader:

    def __init__(self,mail=None,menuw=None):
        rc.app(self)
        self.mail          = mail
        self.menuw         = menuw
        self.mail.v_offset = 0
        self.mail.a_offset = 0
        self.mail.view     = 0

        skin.draw('mailreader', self.mail)

    def toggle_view(self):
        if len(self.mail.MediaAttach)>0:
            if self.mail.view == 0: self.mail.view = 1
            else:                   self.mail.view = 0

    def eventhandler(self, event, menuw=None):

        if config.MENU_ARROW_NAVIGATION:
            if event   == MENU_LEFT:  event = MENU_BACK_ONE_MENU
            elif event == MENU_RIGHT: event = MENU_CHANGE_STYLE
        else:
            if event   == MENU_LEFT:  event = MENU_PAGEUP
            elif event == MENU_RIGHT: event = MENU_PAGEDOWN

        if event == MENU_BACK_ONE_MENU:
            rc.app(None)
            self.menuw.show()
            self.menuw.refresh(reload=1)
            return True

        elif event == MENU_UP:
            if self.mail.view == 0:
                self.mail.v_offset -= self.mail.interval
                if self.mail.v_offset < 0:
                    self.mail.v_offset = 0
            else:
                self.mail.a_offset -= 1
                if self.mail.a_offset < 0:
                    self.mail.a_offset = 0

            skin.draw( 'mailreader', self.mail )
            return True

        elif event == MENU_DOWN:
            if self.mail.view == 0:
                self.mail.v_offset += self.mail.interval
                if self.mail.v_offset>self.mail.max_y:
                    self.mail.v_offset=self.mail.max_y
            else:
                self.mail.a_offset += 1
                if self.mail.a_offset==len(self.mail.MediaAttach):
                    self.mail.a_offset = len(self.mail.MediaAttach) - 1

            skin.draw( 'mailreader', self.mail )
            return True

        elif event == MENU_PAGEUP:
            if self.mail.view == 0:
                self.mail.v_offset -= self.mail.step
                if self.mail.v_offset<0:
                    self.mail.v_offset=0
            else:
                self.mail.a_offset -= 1
                if self.mail.a_offset < 0:
                    self.mail.a_offset = 0

            skin.draw( 'mailreader', self.mail )
            return True

        elif event == MENU_PAGEDOWN:
            if self.mail.view == 0:
                self.mail.v_offset += self.mail.step
                if self.mail.v_offset>self.mail.max_y:
                    self.mail.v_offset=self.mail.max_y
            else:
                self.mail.a_offset += 1
                if self.mail.a_offset == len(self.mail.MediaAttach):
                    self.mail.a_offset = len(self.mail.MediaAttach) - 1

            skin.draw( 'mailreader', self.mail )
            return True

        elif event == MENU_CHANGE_STYLE:
            self.toggle_view()
            skin.draw( 'mailreader', self.mail )
            return True
        elif event == MENU_SELECT:
            if hasattr(config, 'MAIL_SMTP'):
                MailSender(self.mail, self.menuw)
            return True

        return False



class Header(skin.Area):
    """
    the mail header
    """
    def __init__(self):
        skin.Area.__init__(self, 'header')

        # reader display fonts
        self.sub_font      = skin.get_font('medium0')
        self.small_font    = skin.get_font('small0')

    def update_content(self):
        self.header = self.menu
        self.content = self.calc_geometry(self.layout.content, copy_object=True)

        x = self.content.x + 10
        y = self.content.y + 5

        maxw = self.content.width - 20

        # draw Subject
        # for some reason boders and shadows is not
        # compancated in font.height :/
        self.drawstring(self.header.Subject,self.sub_font,
                        self.content, x, y,
                        height=(self.sub_font.height+2),
                        width=maxw, mode='hard')

        y    += self.sub_font.height + 2
        x    += 20
        maxw -= 20

        # draw From
        s_from  = ''
        if self.header.FromName!=_('no name found'):
            s_from += self.header.FromName
            if self.header.FromAddr!=self.header.FromName:
                s_from += Unicode(' - ') + self.header.FromAddr
        else:
            s_from = self.header.FromAddr

        self.drawstring(s_from,self.small_font,
                        self.content, x, y,
                        height=self.small_font.height+2,
                        width=maxw, mode='hard')



class Body(skin.Area):
    """
    The skin area for drawing body text and images.
    This is bit dirty at the moment, as it utilizes
    both osd AND skin code (but it works :).

    Todo:
     o Show arrows for scrolling?
    """
    mail    = None
    int_pad = 10

    def __init__(self):
        skin.Area.__init__(self, 'body')
        self.text_font = skin.get_font('small0')
        self.msg_font  = skin.get_font('small1')
        self.update_functions = (self.update_mail, self.update_attach)

        self.UPARROW   = os.path.join(config.ICON_DIR, 'up.png')
        self.DOWNARROW = os.path.join(config.ICON_DIR, 'down.png')


    def set_mailsurface(self, x, y, w, h, text):
        # update the fonts in case the skin has changed
        self.text_font = skin.get_font('small0')
        self.msg_font  = skin.get_font('small1')

        ### calculate sizes for surface
        self.s_w = w - (2*self.int_pad)
        (x0,y0,x1,y1) = osd.drawstringframed(text, 0, 0, self.s_w, 1000000, self.text_font,
                                     mode='soft', layer='')[1]
        self.s_h = y1 - y0


        # origins
        self.x = x + self.int_pad
        self.y = y + self.int_pad
        self.w = w - (2*self.int_pad)
        self.h = h - (2*self.int_pad)

        # offsets
        self.mail.interval = self.text_font.height
        self.mail.step     = self.content.height
        self.mail.max_y    = (self.s_h - h) + (2*self.int_pad)

        ### create the textual surface, and draw the text
        self.txtSurface = pygame.Surface((self.s_w, self.s_h), pygame.constants.SRCALPHA, 32)
        osd.drawstringframed(text, 0, 0, self.s_w, self.s_h, self.text_font,
                             mode='soft', layer=self.txtSurface)

    def scale_image(self, image):
        """
        Scale image, respect the aspect ratio
        """
        p = pygame.image.load(image).convert_alpha()
        w, h = p.get_size()

        if h>self.h or w>self.w:
            aspect = ( float(w) / float(h) )
            if h>self.h:
                w = int( ( float(self.h) * aspect ) )
                h = self.h
            if w>self.w:
                h = int( ( float(self.w) / aspect ) )
                w = self.w
            p = pygame.transform.scale(p,(w,h))

        return p,w,h

    def update_attach(self):

        image = self.mail.MediaAttach[self.mail.a_offset]
        img,w,h = self.scale_image(image[0])

        # center image on surface
        x0 = (self.w - w)/2 + self.x
        y0 = (self.h - h)/2 + self.y

        self.drawimage(img, (x0, y0, w, h) )

        att = _('Attachment %i/%i: %s') % (self.mail.a_offset+1,
                                           len(self.mail.MediaAttach),
                                           image[1])

        self.drawstring(att, self.msg_font, self.content, self.content.x+10,
                        self.content.y+self.content.height + 10,
                        height=self.msg_font.height,
                        mode='hard')
        # draw arrows
        x  = self.content.x + self.content.width - 25
        y0 = self.content.y + 5
        y1 = self.content.y + self.content.height - 25

        if self.mail.a_offset>0:
            self.drawimage(self.UPARROW,   ( x, y0, 20, 20))
        if self.mail.a_offset<(len(self.mail.MediaAttach)-1):
            self.drawimage(self.DOWNARROW, ( x, y1, 20, 20))

    def update_mail(self):

        if self.mail != self.menu:
            self.mail = self.menu

            self.content = self.calc_geometry(self.layout.content,
                                              copy_object=True)

            self.set_mailsurface(self.content.x,     self.content.y,
                                 self.content.width, self.content.height,
                                 self.mail.BodyText)

        if self.s_h<self.h:
            self.mail.v_offset = 0

        rSurface = pygame.Surface((self.s_w, self.s_h), pygame.constants.SRCALPHA, 32)

        # blit txtsurface and draw it as an image
        rSurface.blit(self.txtSurface,(0,0),
                      (0, self.mail.v_offset, self.w, self.h) )
        self.drawimage(rSurface,
                       ( self.x, self.y, self.w, self.h) )

        msg = ''
        if len(self.mail.MediaAttach)>0:
            if config.MENU_ARROW_NAVIGATION:
                msg = _('Right: View image attachment(s). ')
            else:
                msg = _('Display: View image attachment(s). ')

        if hasattr(config, 'MAIL_SMTP'):
            msg += _('Select: Reply to message.')


        self.drawstring(msg, self.msg_font, self.content, self.content.x + 10,
                        self.content.y+self.content.height + 10,
                        height=self.msg_font.height,
                        mode='hard')

        # draw arrows
        x  = self.content.x + self.content.width - 25
        y0 = self.content.y + 5
        y1 = self.content.y + self.content.height - 25

        if self.mail.v_offset>0:
            self.drawimage(self.UPARROW,   ( x, y0, 20, 20))
        if self.mail.v_offset<self.mail.max_y:
            self.drawimage(self.DOWNARROW, ( x, y1, 20, 20))

    def update_content(self):
        self.update_functions[self.menu.view]()


skin.register ( 'mailreader', ('screen', Body(), Header(), 'plugin'))

