#if 0 /*
# -----------------------------------------------------------------------
# mbox.py - UNIX mbox implementation
# Author: Viggo Fredriksen <viggo@katatonic.org>
# -----------------------------------------------------------------------
# $Id: mbox.py,v 1.8 2004/05/19 10:23:34 viggo Exp $
# -----------------------------------------------------------------------
# $Log: mbox.py,v $
# Revision 1.8  2004/05/19 10:23:34  viggo
# Fix Id and Log handling in sourcefiles
#
#
# -----------------------------------------------------------------------
# Freevo - A Home Theater PC framework
# Copyright (C) 2003 Krister Lagerstrom, et al.
# Please see the file freevo/Docs/CREDITS for a complete list of authors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MER-
# CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# ----------------------------------------------------------------------- */
#endif

# python modules
import mailbox, email, os, stat

# freevo modules
import config

# mailreader modules
from mail     import MailProtocol
from mailutil import *



class Mbox(MailProtocol):
    """
    UNIX mbox implementation
    spec: http://www.qmail.org/man/man5/mbox.html
    doc:  http://www.python.org/doc/current/lib/module-mailbox.html
    """

    def __init__(self):
        MailProtocol.__init__(self)
        self.uidlist = []
        self.caps = ['DEL']
        self.fdict = {}

    def _getBox(self, mbox, factory=1):
        if factory==1:
            return mailbox.PortableUnixMailbox(file(mbox,'rb'), self._msgfactory)
        elif factory==2:
            return mailbox.PortableUnixMailbox(file(mbox,'rb'), self._delfactory)
        elif factory==3:
            return mailbox.PortableUnixMailbox(file(mbox,'rb'), self._headerfactory)

    def _msgfactory(self,fp):
        try:
            return email.message_from_file(fp)
        except email.Errors.MessageParseError:
            return ''

    def _headerfactory(self,fp):
        try:
            return email.Parser.HeaderParser().parsestr(fp.read(), headersonly=1)
        except:
            return ''

    def _delfactory(self,fp):
        try:
            pos = fp.tell()
            strmsg = fp.read()
            fp.seek(pos)
            msg = email.message_from_file(fp)
            return msg,strmsg
        except email.Errors.MessageParseError:
            return '', ''

    def getUpdates(self, folder, seen):
        upd   = []
        rem   = []
        add   = []

        mtime = os.stat(folder)[stat.ST_MTIME]
        if mtime!=self.fdict[folder]:
            add,rem,uidl = self.getHeaders(folder,seen)
            self.Sort(add)
        else:
            uidl = seen

        return upd,rem,add,uidl

    def getHeader(self,folder,uid):
        mbox = self._getBox(folder,3)

        header = BaseHeader()
        msg = mbox.next()

        while msg:
            header = getHeader(msg,message=False)
            if header.UID == uid:
                header.Flag  = 'new'
                header.Folder = folder
                return header
            msg = mbox.next()

        return header

    def getHeaders(self,folder,seen=None):
        """
        I have no way of knowing flags for this
        protocol? (new,etc)
        """
        mbox = self._getBox(folder,3)
        mails = []
        uidlist = []

        self.fdict[folder] = os.stat(folder)[stat.ST_MTIME]

        msg = mbox.next()

        i = 0
        while msg:
            header = getHeader(msg,message=False)
            header.Flag = 'new' # no way of knowing?
            header.Folder = folder
            mails.append(header)
            uidlist.append( (header.UID,i) )
            msg = mbox.next()
            i += 1

        # update internals
        if seen:
            add  = []
            rem  = []
            uidl = []
            for (id,i) in uidlist:
                uidl.append(id)
                if id not in seen:
                    add.append(mails[i])
            for id in seen:
                if id not in uidl:
                    rem.append(id)
            return add,rem,uidl

        return self.Sort(mails)

    def getMessage(self,header):
        mbox = self._getBox(header.Folder,1)
        msg = mbox.next()

        while msg['Message-ID']!=header.UID and msg is not None:
           msg = mbox.next()

        if msg:
            return getMessage(msg,header,1)
        else:
            return BaseMessage(header)

    def delMessage(self, header):
        outputstr = ''
        mbox      = self._getBox(header.Folder, 2)
        msg       = mbox.next()

        while msg:
            m,strmsg = msg
            if m['Message-ID']!=header.UID:
                outputstr += strmsg
            msg = mbox.next()

        try:
            fp = file(header.Folder,'wb+')
            fp.write(outputstr)
            fp.flush()
            fp.close()
            return True
        except Exception, e:
            return False
